@@ -1,8 +1,13 @@
-# $Id: Block.pm,v 1.9 2001/07/27 16:26:15 rcaputo Exp $
+# $Id: Block.pm,v 1.11 2002/07/31 20:05:20 rcaputo Exp $
 
 package POE::Filter::Block;
+use POE::Preprocessor ( isa => "POE::Macro::UseBytes" );
 
 use strict;
+
+use vars qw($VERSION);
+$VERSION = (qw($Revision: 1.11 $ ))[1];
+
 use Carp qw(croak);
 
 sub BLOCK_SIZE     () { 0 }
@@ -38,6 +43,8 @@ sub get {
   my @blocks;
   $self->[FRAMING_BUFFER] .= join '', @{$stream};
 
+  {% use_bytes %}
+
   # If a block size is specified, then frame input into blocks of that
   # size.
   if (defined $self->[BLOCK_SIZE]) {
@@ -82,8 +89,10 @@ sub get_one_start {
 sub get_one {
   my $self = shift;
 
+  {% use_bytes %}
+
   # If a block size is specified, then pull off a block of that many
-  # bytes and/or characters, depending upon what length() does.
+  # bytes.
 
   if (defined $self->[BLOCK_SIZE]) {
     return [ ] unless length($self->[FRAMING_BUFFER]) >= $self->[BLOCK_SIZE];
@@ -118,6 +127,8 @@ sub put {
   my ($self, $blocks) = @_;
   my @raw;
 
+  {% use_bytes %}
+
   # If a block size is specified, then just assume the put is right.
   # This will cause quiet framing errors on the receiving side.  Then
   # again, we'll have quiet errors if the block sizes on both ends
