@@ -1,11 +1,11 @@
-# $Id: ReadWrite.pm,v 1.61 2002/11/20 17:57:41 rcaputo Exp $
+# $Id: ReadWrite.pm,v 1.62 2003/05/02 12:36:20 rcaputo Exp $
 
 package POE::Wheel::ReadWrite;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = (qw($Revision: 1.61 $ ))[1];
+$VERSION = (qw($Revision: 1.62 $ ))[1];
 
 use Carp;
 use POE qw(Wheel Driver::SysRW Filter::Line);
@@ -606,6 +606,24 @@ sub resume_input {
   }
 }
 
+# Shutdown the socket for reading.
+sub shutdown_input {
+  my $self = shift;
+  if (defined $self->[HANDLE_INPUT]) {
+    eval { local $^W = 0; shutdown($self->[HANDLE_INPUT], 0) };
+    $poe_kernel->select_read($self->[HANDLE_INPUT], undef);
+  }
+}
+
+# Shutdown the socket for writing.
+sub shutdown_output {
+  my $self = shift;
+  if (defined $self->[HANDLE_OUTPUT]) {
+    eval { local $^W=0; shutdown($self->[HANDLE_OUTPUT], 1) };
+    $poe_kernel->select_write($self->[HANDLE_OUTPUT], undef);
+  }
+}
+
 ###############################################################################
 1;
 
@@ -676,6 +694,10 @@ POE::Wheel::ReadWrite - buffered non-blocking I/O
   $wheel->pause_input();
   $wheel->resume_input();
 
+  # To shutdown a wheel's socket(s).
+  $wheel->shutdown_input();
+  $wheel->shutdown_output();
+
 =head1 DESCRIPTION
 
 ReadWrite performs buffered, select-based I/O on filehandles.  It
@@ -768,6 +790,14 @@ corresponding output buffer) from being overwhelmed.
 resume_input() instructs the wheel to resume checking its input
 filehandle for data.
 
+=item shutdown_input
+
+=item shutdown_output
+
+Some applications require the remote end to shut down a socket before
+they will continue.  These methods map directly to shutdown() for the
+wheel's input and output sockets.
+
 =back
 
 =head1 EVENTS AND PARAMETERS
